#!/usr/bin/env node
const path = require('path')
const assert = require('assert')
const fs = require('fs-extra')
const program = require('commander')
const server = require('../app/server')

program
  .option('--client <path>', 'client location', 'build')
  .option('--server <path>', 'src location', 'server')
  .option('--middleware <path>', 'src location', 'middleware.js')
  .option('--port <n>', 'server port', 8080)
  .parse(process.argv)

const port = parseInt(program.port, 10)
const cwd = process.cwd()

const configFile = program.args.slice().pop() || 'package.json'
let userConfig = {}
try {
  const config = JSON.parse(fs.readFileSync(path.join(cwd, configFile), 'utf8'))
  if (config['server']) {
    userConfig = config['server']
  }
} catch (e) {}

const config = {
  ...{
    client: program.client,
    server: program.server,
    middleware: program.middleware
  },
  ...userConfig
}

// get middleware generator
const serverDirectory = path.join(cwd, config.server)
const middlewareLocation = path.join(serverDirectory, config.middleware)
const clientDirectory = path.join(cwd, config.client)

let middleware
let html

try {
  middleware = require(middlewareLocation)
  html = fs.readFileSync(path.join(clientDirectory, 'index.html'), 'utf8')
} catch (e) {
  middleware = require(path.join(__dirname, '../app/default-middleware'))
  html = fs.readFileSync(path.join(__dirname, '../app/default-index.html'), 'utf8')
  console.warn(e)
}

// check if we are dealing with an expected middleware object
const {
  create,
  destroy = () => {},
  extensions = []
} = middleware
assert(typeof create === 'function', 'Middleware exports a create() method')

// handle extensions
const applyMiddleware = extensions && extensions.length
  ? app => extensions.forEach(extension => extension(app))
  : null

// start the server
server({
  port,
  html,
  ...userConfig,
  createMiddleware: create,
  destroyMiddleware: destroy,
  applyMiddleware,
  webroot: clientDirectory
})
